import React, { useState, useEffect, useRef, useMemo } from 'react'
import { Input, Button, Tree, Spin, Table } from 'antd'
import { CompassOutlined, DownOutlined } from '@ant-design/icons'
import styles from './index.less'
import {
  ArcGISSceneMap,
  AutoCompleteSearch,
  Drawtool as drawTool,
  Graphic,
  GraphicsLayer,
  Point,
  geomUtils,
  SimpleLineSymbol,
  SimpleFillSymbol,
} from '@wisdom-map/arcgismap'
import Drag from '../../../components/Drag'
import { isJson, isObject, getNanoid, mercatorToLngLat } from '../../../../utils'
import { FetchAreaList, GetAreaLayerTaskData, GetAccountConfigInfo, GetAccountPageList } from '../../../../apis/process'

const AreaTask = (props) => {

  const codes = window?.pandaXform?.codes || { 工单编号: '', 事件编号: '', }
  const userID = window?.globalConfig?.userInfo?.OID || 1;
  const { token, client } = window.globalConfig
  const { value, onChange, schema, addons } = props
  const {
    titleShow,
    disabled,
    placeholder,
    presetValue,
    addressSync,
    screenShot,
    service,
    layerName,
    areaTaskShine,
    accountName,
  } = schema
  const [visible, setVisible] = useState(false)
  const [rings, setRings] = useState({})
  const [text, setText] = useState(presetValue ? '已绘制' : '（空）')
  const [put, setPut] = useState(true)
  const [tab, setTab] = useState('一级')
  const [areaList, setAreaList] = useState([])
  const [selectedKeys, setSelectedKeys] = useState([])
  const [loading, setLoading] = useState(false)
  const [config, setConfig] = useState({ accountFieids: [], formJson: '' })
  const { accountFieids, formJson } = config
  const [dataSource, setDataSource] = useState([])
  const viewRef = useRef(null)
  const layers = useRef(null)

  const treeData = useMemo(() => {
    const ListToTree = (List, rootValue) => {
      const treeData = []
      if (List.length) {
        List.forEach(v => {
          if (v.parentID === rootValue) {
            const children = ListToTree(List, v.id)
            treeData.push({ key: v.id, title: v.name, parentID: v.parentID, areaPolygon: v.areaPolygon, children })
          }
        })
      }
      return treeData
    }
    const treeData = ListToTree(areaList, 0)
    if (tab === '二级') {
      let array = []
      treeData.forEach(v => {
        v.children.forEach(s => {
          array.push({ ...s })
        })
      })
      return array
    }
    if (tab === '三级') {
      let array = []
      treeData.forEach(v => {
        v.children.forEach(s => {
          s.children.forEach(j => {
            array.push({ ...j })
          })
        })
      })
      return array
    }
    return treeData
  }, [areaList, tab])

  const columns = useMemo(() => {
    let array = []
    if (accountFieids.length) {
      accountFieids.forEach(v => {
        if (v.webDisplay === 1) {
          array.push({ title: v.fieldName, dataIndex: v.fieldName, key: v.fieldName })
        }
      })
    } else {
      areaTaskShine?.forEach(v => {
        array.push({ title: v.toField, dataIndex: v.toField, key: v.toField })
      })
    }
    return array
  }, [areaTaskShine, accountFieids])

  const onSelect = (value, areaList) => {
    setSelectedKeys(value)
    let areaPolygon = areaList.find(v => v.id === value?.[0])?.areaPolygon
    if (areaPolygon) {
      let geometry = geomUtils.toGeometry(JSON.parse(areaPolygon))
      setRings({ rings: geometry.rings })
      let layer = new Graphic({
        geometry: geometry,
        symbol: new SimpleFillSymbol({
          // color: [83, 164, 253, 0.5],
          // outline: {
          //   color: [255, 255, 255, 1],
          //   width: 2,
          // },
        }),
      })
      viewRef.current.goTo({ target: geometry, zoom: viewRef.current.zoom > 20 ? 20 : viewRef.current.zoom > 15 ? viewRef.current.zoom : 15 }, { duration: 500 })
      if (layers?.current) {
        viewRef?.current?.map.remove(layers.current)
      }
      layers.current = new GraphicsLayer()
      layers.current.add(layer)
      viewRef.current.map.add(layers.current)
    }
  }

  const createPaths = (view) => {
    setTimeout(() => {
      if (isJson(value)) {
        let layer = new Graphic({
          geometry: geomUtils.toGeometry(JSON.parse(value)),
          symbol: new SimpleFillSymbol({

          }),
        })
        layers.current = new GraphicsLayer()
        layers.current.add(layer)
        view.map.add(layers.current)
      }
    }, 2000)
  }

  const getView = (viewObject) => {
    setRings(isJson(value) ? JSON.parse(value) : '')
    viewRef.current = viewObject
    createPaths(viewObject)
  }

  const getMapInfo = (viewObject) => {
    setRings(isJson(value) ? JSON.parse(value) : '')
    viewRef.current = viewObject
    createPaths(viewObject)
  }

  const getPoints = (view) => {
    if (view) {
      view.map.remove(layers.current)
      drawTool.activate({
        view,
        action: 'polygon',
        target: 'erroSendUpWidget',// 随便写
        toolTip: '左键选择位置',
        drawEnd: geometry => {
          setRings({ rings: geometry.rings })
          let layer = new Graphic({
            geometry: geometry,
            symbol: new SimpleFillSymbol({

            }),
          })
          layers.current = new GraphicsLayer()
          layers.current.add(layer)
          view.map.add(layers.current)
        },
        // rightClick: () => { getPoints(view, coordGetLayer) }
      })
    }
  }

  const start = () => {
    setSelectedKeys([])
    getPoints(viewRef.current)
  }

  const showMap = () => {
    getData(value)
    setVisible(true)
  }

  const onCancel = () => {
    setVisible(false)
  }

  const getGISData = async () => {
    window.relationForm = { configs: [], data: [] }
    let layerWhere = []
    if (layerName?.length) {
      layerName.forEach(v => {
        layerWhere.push({ layerName: v, where: [] })
      })
    }
    let paramas = {
      mapServerName: service,
      geometry: JSON.stringify(rings),
      geometryType: 'civGeometryPolygon',
      layerWhereList: layerWhere
    }
    const { code, data } = await GetAreaLayerTaskData(paramas)
    if (code === 0) {
      let items = []
      let array = []
      data.forEach(v => {
        let obj = {
          ID: `${getNanoid(10, '前端ID')}`
        }
        let values = []
        areaTaskShine.forEach(s => {
          if (s.fromField === '工单编号') {
            obj[s.toField] = codes['工单编号']
          } else if (s.fromField === 'GIS图层') {
            obj[s.toField] = v['gis图层'] || ''
          } else if (s.fromField === 'GIS编号') {
            obj[s.toField] = v['gis编号'] || ''
          } else {
            obj[s.toField] = v[s.fromField] || ''
          }
          values.push({ fieldName: s.toField, fieldValue: obj[s.toField] })
        })
        array.push(obj)
        items.push({
          type: 'add',
          accountTable: accountName,
          ID: `${getNanoid(10, '前端ID')}`,
          values: values
        })
      })
      window.relationForm = { configs: [{ ...schema, '台账名称': schema.accountName, '映射字段': [{ fromField: '工单编号', toField: '工单编号' }] }], data: items }
      setDataSource(array)
    }
  }

  const onOk = () => {
    if (selectedKeys.length) {
      onChange(`${selectedKeys[0]}`)
    } else {
      onChange(JSON.stringify(rings))
    }
    getGISData()
    setVisible(false)
  }

  const getData = async (value) => {
    const { code, data } = await FetchAreaList({ userID: userID, sourceType: '管网巡检,DMA分区' })
    if (code === '0') {
      setAreaList(data)
      if (value && !isNaN(value)) {
        setTimeout(() => {
          onSelect([Number(value)], data)
        }, 2000)
      }
    }
  }

  const getConfig = async () => {
    const { code, data } = await GetAccountConfigInfo(accountName)
    if (code === 0) {
      setConfig(data)
    }
  }

  const getDataSource = async (presetValue) => {
    let params = {
      user: userID,
      condition: '',
      accountName: accountName,
      sortFields: '录入时间',
      direction: 'desc',
      timeField: '录入时间',
      pageIndex: 0,
      pageSize: 0,
      queryWheres: [{ field: '工单编号', type: '等于', value: codes['工单编号'] }],
    }
    if (!codes['工单编号']) return
    const { code, data, msg } = await GetAccountPageList(params)
    if (code === 0) {
      let tableData = data?.jsonData ? JSON.parse(data?.jsonData) : []
      setDataSource(tableData)
    }
  }

  useEffect(() => {
    if (addons) {
      addons.setValueByPath(addons.dataPath, presetValue)
      if (presetValue) {
        getConfig()
        getDataSource(presetValue)
      }
    } else {
      onChange(presetValue)
    }
  }, [presetValue])

  useEffect(() => {
    if (disabled && screenShot) {
      addons?.setSchemaByPath(addons.dataPath, { ...schema, width: '67%' })
    }
  }, [disabled, screenShot])


  if (screenShot && disabled && addons) {
    return (
      <div className={styles.drawAreaShot}>
        <div className={styles.content}>
          <div className={styles.top}>
            <div className={styles.text}>{text}</div>
            <div
              className={styles.icon}
              onClick={() => setPut(!put)}
              style={{ transform: `rotate(${put ? 0 : 180}deg)` }}
            >
              <DownOutlined />
            </div>
          </div>
          <div className={styles.mapBox} style={{ height: put ? '208px' : '0px' }}>
            <ArcGISSceneMap
              getMapInfo={getMapInfo}
              widgets={[]}
              token={token}
              client={token ? client : 'sandbox'}
            />
          </div>
        </div>
      </div>
    )
  }

  return (
    <div className={styles.AreaTask}>
      <div className={styles.areaTitle}>{titleShow}</div>
      <div className={styles.areaBox}>
        <div className={styles.areaInput}>
          {
            !disabled ?
              <Input
                disabled={disabled}
                placeholder={disabled ? (placeholder || '') : (placeholder || '点击绘制区域')}
                value={isJson(value) ? '已绘制' : (value ? '已选择' : '')}
                addonAfter={disabled ? null : <CompassOutlined style={{ color: 'rgba(0, 0, 0, 0.25)' }} onClick={showMap} />}
                style={{ width: '100%' }}
                onClick={() => showMap()}
              />
              :
              <Button
                icon={<CompassOutlined style={{ color: value ? '#0092fe' : 'rgba(0, 0, 0, 0.25)' }} />}
                onClick={() => {
                  if (value) {
                    showMap()
                  }
                }}
              >
                {value ? '查看区域' : '无区域信息'}
              </Button>
          }
        </div>
        <div className={styles.areaTable}>
          <Table
            size='small'
            rowKey='ID'
            bordered
            columns={columns}
            dataSource={dataSource}
          />
        </div>
      </div>
      <Drag
        width={'80%'}
        title="选取区域"
        visible={visible}
        onOk={onOk}
        onCancel={onCancel}
        cancelText={'取消'}
        okText={'确定'}
        bodyStyle={{ height: 600, padding: '0' }}
        destroyOnClose={true}
        {...disabled ? { footer: null } : {}}
      >
        <div className={styles.AreaTaskMap}>
          <Spin spinning={loading} tip="获取GIS数据中...">
            <div className={styles.content}>
              <div className={styles.left}>
                {/* <div className={styles.tabs}>
                  {
                    ['一级', '二级', '三级'].map(v => <div className={styles.tab} active={`${v === tab}`} key={v} onClick={() => setTab(v)}>{v}</div>)
                  }
                </div> */}
                <div className={styles.tabContent}>
                  <Tree
                    defaultExpandAll
                    blockNode
                    selectedKeys={selectedKeys}
                    onSelect={(value) => onSelect(value, areaList)}
                    treeData={treeData}
                  />
                </div>
              </div>
              <div className={styles.right}>
                <div>
                  <ArcGISSceneMap
                    getMapInfo={getView}
                    widgets={[]}
                    token={token}
                    client={token ? client : 'sandbox'}
                  />
                </div>
              </div>
              <Button
                style={{ position: 'absolute', zIndex: '99', right: '10px', top: '10px' }}
                type='primary'
                onClick={start}
              >
                开始绘制
              </Button>
            </div>
          </Spin>
        </div>
      </Drag>
    </div>
  )

}

export default AreaTask